/**
******************************************************************************
* @file    IRPyro_UT.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   Low level Unit Testing module
******************************************************************************
  @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>

  @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
******************************************************************************

*/
/* Includes ------------------------------------------------------------------*/
#include "IRPyro_UT.h"
/* Unit Test Private variables -----------------------------------------------*/
/* Unit Test Private function prototypes--------------------------------------*/
/*--------------------- TEST FRAME COUNTING ----------------------------------*/
static uint8_t IRPyro_test_single_sensor_read_sample(IRPyro_sensor_device_type* this_device);
static uint8_t IRPyro_test_multiple_sensor_read_sample(IRPyro_devices this_list, uint8_t list_size);
/* Unit Test Support function prototypes--------------------------------------*/
static uint8_t AFE_compare(AFE_reg_type first, AFE_reg_type second);
static uint8_t CCP_compare(CCP_reg_type first, CCP_reg_type second);
static uint8_t WUP_compare(WUP_reg_type first, WUP_reg_type second);
static uint8_t check_results(uint8_t * test_results, uint8_t number_of_tests);
static uint8_t FIFO_reset(IRPyro_sensor_device_type* this_device);
static void wakeup_event_callback(void);

/** @defgroup IRPyroAPI_UnitTest Unit Test Module
 * @brief Implements the general case of each unit test description
 * @verbatim
 *    Functions to validate and test the IRPyro API implementation
 *    Each function covers a point of the product verification plan
 * @endverbatim
 * @{
 */
/** @defgroup UT_02_IRPyro 9.6.1 Single Sensor initialization
 * @brief Verification of single sensor initialization in multiple modes
 <table>
 <tr><td> 9.6.1.1 </td><td> Initialize a single device for interrupt configuration   </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory.</td></tr>
 <tr><td> 9.6.1.2 </td><td> Initialize a single device for polled configuration      </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory.</td></tr>
 <tr><td> 9.6.1.3 </td><td> Initialize a single device for synchronized configuration </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory.</td></tr>
 </table>
 * @{
 */
uint8_t ut01_single_sensor_init(IRPyro_sensor_device_type *IRPyro_test_Sensor)
{
    uint8_t op_result = 0;
    uint8_t test_results[10]= {0};
    IRPyro_cmd_type IOCTL_cmd = {0};
    IRPyro_arg_type IOCTL_arg = {0};

    UART_PutString(USART2, (uint8_t*)"UT_01_IRPyro 9.6.1 Single Sensor initialization \n");
    /*-------------------------- start test ----------------------------------*/
    IOCTL_cmd.cmd = register_get;

    test_results[0] = IRPyro_open(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"OPEN SENSOR \n");
    test_results[1] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    UART_PutString(USART2, (uint8_t*)"AFE compare \n");
    test_results[2] =AFE_compare(IRPyro_test_Sensor->AFE_register, IOCTL_cmd.AFE_register);
    UART_PutString(USART2, (uint8_t*)"CCP compare \n");
    for(uint8_t idx=0; idx<IRPyro_CCP_SIZE; ++idx)
    {
        test_results[3+idx]=CCP_compare(IRPyro_test_Sensor->CCP_register[idx], IOCTL_cmd.CCP_register[idx]);
    }
    UART_PutString(USART2, (uint8_t*)"WUP compare \n");
    test_results[8]=WUP_compare(IRPyro_test_Sensor->WUP_register, IOCTL_cmd.WUP_register);
    test_results[9]=IRPyro_close(IRPyro_test_Sensor);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(test_results, 10);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF SINGLE SENSOR INIT-------------------------\n");
    return op_result;
}
/** end of 9.6.1
 * @}
 */
/** @defgroup UT_03_IRPyro 9.6.2 Multiple Sensor initialization
 * @brief Verification of multiple sensors initialization in multiple modes
 <table>
 <tr><td> 9.6.2.1 </td><td> Initialize multiple devices for interrupt configuration   </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory. </td></tr>
 <tr><td> 9.6.2.2 </td><td> Initialize multiple devices for polled configuration      </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory. </td></tr>
 <tr><td> 9.6.2.3 </td><td> Initialize multiple devices for synchronized configuration </td></tr>
 <tr><td>         </td><td> Written configuration matches registers stored on memory. </td></tr>
 </table>
 * @{
 */
uint8_t ut02_multiple_sensor_init(IRPyro_devices this_list, uint8_t list_size)
{
    uint8_t op_result = 0;
    uint8_t test_results[2]     = {0};
    UART_PutString(USART2, (uint8_t*)"UT_02_IRPyro 9.6.2 Multiple Sensor initialization \n");
    /*-------- test the open_list, read_list and close_list functions --------*/
    /*-------------------------- open_list   ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"OPEN devices \n");
    test_results[0] = IRPyro_open_devices(this_list, list_size);
    /*-------------------------- close_list  ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"CLOSE devices \n");
    test_results[1] = IRPyro_close_devices(this_list, list_size);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(test_results, 2);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF MULTIPLE SENSOR INIT-------------------------\n");
    return op_result;
}
/** end of 9.6.2
 * @}
 */
/** @defgroup UT_04_IRPyro 9.6.3 Single Sensor Data Read
 * @brief Single sensor data reading with frame detection
 <table>
 <tr><td> 9.6.3.1 </td><td> Read data from a single sensor and detect frames missing with the sensor configured for interrupt.  </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                   </td></tr>
 <tr><td> 9.6.3.2 </td><td> Read data from a single sensor and detect frames missing with the sensor configured for polled      </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                   </td></tr>
 <tr><td> 9.6.3.3 </td><td> Read data from a single sensor and detect frames missing with the sensor configured for synchronized </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                   </td></tr>
 </table>
 * @{
 */
uint8_t ut03_single_sensor_read(IRPyro_sensor_device_type *IRPyro_test_Sensor)
{
    uint8_t op_result = 0;
    /*-------------------------- template definition -------------------------*/
    uint8_t test_results[3] = {0};
    UART_PutString(USART2, (uint8_t*)"UT_03_IRPyro 9.6.3 Single Sensor Data Read \n");
    /*-------------------------- TEST BEGINS ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"OPEN device \n");
    test_results[0] = IRPyro_open(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"READING SAMPLE \n");
    test_results[1] = IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"CLOSE device \n");
    test_results[2] = IRPyro_close(IRPyro_test_Sensor);

    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(test_results, 3);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF SINGLE SENSOR READ-------------------------\n");
    return op_result;
}
/** end of 9.6.3
 * @}
 */
/** @defgroup UT_05_IRPyro 9.6.4 Multiple Sensor Data Reading
 * @brief Multiple sensor reading with frame detection
 <table>
 <tr><td> 9.6.4.1 </td><td> Read data from multiple sensors and detect frames missing with the sensors configured for interrupt   </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                     </td></tr>
 <tr><td> 9.6.4.2 </td><td> Read data from multiple sensors and detect frames missing with the sensors configured for polled      </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                     </td></tr>
 <tr><td> 9.6.4.3 </td><td> Read data from multiple sensors and detect frames missing with the sensors configured for synchronized </td></tr>
 <tr><td>         </td><td> No frames should be missing from a sample period.                                                     </td></tr>
 </table>
 * @{
 */
uint8_t ut04_multiple_sensor_read(IRPyro_devices this_list, uint8_t list_size)
{
    uint8_t op_result=0;
    uint8_t test_results[3]     = {0};
    UART_PutString(USART2, (uint8_t*)"UT_04_IRPyro 9.6.4 Multiple Sensor Data Reading \n");
    /*-------- test the open_list, read_list and close_list functions --------*/
    /*-------------------------- open_list   ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"OPEN devices \n");
    test_results[0] = IRPyro_open_devices(this_list, list_size);
    /*-------------------------- read_list   ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"READING devices \n");
    test_results[1] = IRPyro_test_multiple_sensor_read_sample(this_list, list_size);
    /*-------------------------- close_list  ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"CLOSE devices \n");
    test_results[2] = IRPyro_close_devices(this_list, list_size);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(test_results, 2);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF MULTIPLE SENSOR READ-------------------------\n");
    return op_result;
}
/** end of 9.6.4
 * @}
 */
/** @defgroup UT_06_IRPyro 9.6.5 Data collection with NORTH channel enabled
 * @brief Data collection with orientation enabled
 <table>
 <tr><td> 9.6.5 </td><td> Read data from devices with 1 to 4 channels enabled changing NORTH channel designation                  </td></tr>
 <tr><td>       </td><td> Data frame returns ordered                                                                              </td></tr>
 </table>
 * @{
 */
// deprecated
/** end of 9.6.5
 * @}
 */
/** @defgroup UT_07_IRPyro 9.6.6 Power options testing
 * @brief Power options testing
 <table>
 <tr><td>  9.6.6 </td><td> Change sensor power configuration </td></tr>
 <tr><td>        </td><td>                                   </td></tr>
 </table>
 * @{
 */
/**
 * @brief Test of IOCTL functions
 */
uint8_t ut06_IOCTL01(IRPyro_sensor_device_type *IRPyro_test_Sensor)
{
#define IOCTL01_TEST_NUMBER 5
    /*-------------------------- memory allocation----------------------------*/
    uint8_t         op_result        = 0;
    IRPyro_cmd_type IOCTL_cmd = {0};
    IRPyro_arg_type IOCTL_arg = {0};
    uint8_t         IOCTL_result[IOCTL01_TEST_NUMBER] = {0};

    IRPyro_open(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"UT_06_IRPyro 9.6.6 Power options testing \n");

    /*------------------------ Power control ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"POWER NORMAL \n");
    IOCTL_cmd.cmd = power_normal;
    IOCTL_result[0] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"POWER LOW \n");
    IOCTL_cmd.cmd = power_low;
    IOCTL_result[1] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"POWER SLEEP \n");
    IOCTL_cmd.cmd = power_sleep;
    IOCTL_result[2] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);

    IOCTL_cmd.cmd = power_wakeup;
    UART_PutString(USART2, (uint8_t*)"POWER WAKE UP \n");
    IOCTL_result[3] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);

    IOCTL_cmd.cmd = power_off;
    UART_PutString(USART2, (uint8_t*)"POWER OFF \n");
    IOCTL_result[4] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);

    IRPyro_close(IRPyro_test_Sensor);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(IOCTL_result, IOCTL01_TEST_NUMBER);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF MULTIPLE SENSOR READ-------------------------\n");
    return op_result;
};
/** end of 9.6.6
 * @}
 */
/** @defgroup UT_08_IRPyro 9.6.7 Sleep and Wake-up operation
 * @brief Configuration recovery after sleep command and wake-up event
 <table>
 <tr><td>  9.6.7    </td><td> Wake command response             </td></tr>
 <tr><td>        </td><td>                                   </td></tr>
 </table>
 * @{
 */
/**
 * @brief  Wake up event
 */
uint8_t ut_wake_up_event(IRPyro_sensor_device_type *IRPyro_test_Sensor)
{
#define WAKE_UP_EVENT_TEST_NUMBER 3
    /*-------------------------- memory allocation----------------------------*/
    uint8_t         op_result        = 0;
    uint8_t         wake_up_event_result[WAKE_UP_EVENT_TEST_NUMBER] = {0};
    uint32_t        time_to_wait     = TIME_TO_WAIT;
    uint8_t         wake_up_detected = 0;
    IRPyro_cmd_type IOCTL_cmd        = {0};
    IRPyro_arg_type IOCTL_arg        = {0};

    /*------------------------ Begin Test ------------------------------------*/
    IRPyro_open(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"WAKE UP EVENT DETECTION BY HARDWARE INTERRUPT \n");
    /*------------------------ Power control ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"POWER NORMAL \n");
    IOCTL_cmd.cmd = power_normal;
    wake_up_event_result[0] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"POWER SLEEP \n");
    IOCTL_cmd.cmd = power_sleep;
    wake_up_event_result[1] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);

    EXTernal_Callback_Setup(wakeup_event_callback);
    PWR_EnterSleepMode(PWR_SLEEPEntry_WFI);

    /*wait for the sensor to wake up or time to run out*/
    do
    {
        wake_up_detected = EXTI_detected(0);
        --time_to_wait;
    }
    while ((time_to_wait != 0) && (wake_up_detected == 0));

    if (time_to_wait == 0 && wake_up_detected == 0)
    {
        IOCTL_cmd.cmd = power_wakeup;
        UART_PutString(USART2, (uint8_t*)"FORCED POWER WAKE UP \n");
        IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
        UART_PutString(USART2, (uint8_t*)"READ TEST AFTER FORCED POWER WAKE UP \n");
        IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"WAKE UP EVENT DETECTED!! \n");
        wake_up_event_result[2] = wake_up_detected;
        UART_PutString(USART2, (uint8_t*)"READ TEST AFTER WAKE UP EVENT \n");
        IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);
    }

    IRPyro_close(IRPyro_test_Sensor);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(wake_up_event_result, WAKE_UP_EVENT_TEST_NUMBER);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------WAKE UP EVENT DETECTION BY HARDWARE INTERRUPT-------------------------\n");
    return op_result;
}
/** end of 9.6.7
 * @}
 */

/** @defgroup UT_09_IRPyro 9.6.8 On the fly configuration
 * @brief IRPyro configuration parameters can be changed on the fly
 <table>
 <tr><td>  9.6.8    </td><td> Change parameters on the fly      </td></tr>
 <tr><td>        </td><td>                                   </td></tr>
 </table>
 * @{
 */
uint8_t ut06_IOCTL02(IRPyro_sensor_device_type *IRPyro_test_Sensor)
{
#define IOCTL_TEST_NUMBER 12
    /*-------------------------- memory allocation----------------------------*/
    uint8_t         op_result        = 0;
    IRPyro_cmd_type IOCTL_cmd = {0};
    IRPyro_arg_type IOCTL_arg = {0};
    uint8_t         IOCTL_result[IOCTL_TEST_NUMBER] = {0};

    IRPyro_open(IRPyro_test_Sensor);
    UART_PutString(USART2, (uint8_t*)"UT_06_IRPyro 9.6.8 On the fly configuration \n");

    /*------------------------ Sampling rate ---------------------------------*/
    UART_PutString(USART2, (uint8_t*)"SAMPLING RATE 1MS \n");
    IOCTL_cmd.cmd = sampling_rate_get;  // Normal power, 0x00 = 1ms
    IOCTL_result[0] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"SAMPLING RATE 10MS \n");
    IOCTL_cmd.cmd  = sampling_rate_set; // Normal power, 0xaa = 10 ms
    IOCTL_arg.arg = 0x0A;
    IOCTL_result[1] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"SAMPLING RATE 1MS \n");
    IOCTL_cmd.cmd  = sampling_rate_set; // Normal power, 0x00 = 1 ms
    IOCTL_arg.arg = 0x00;
    IOCTL_result[2] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    /*------------------------ High pass filter ------------------------------*/
    UART_PutString(USART2, (uint8_t*)"SET HIGH PASS FILTER 2HZ \n");
    IOCTL_cmd.cmd = high_pass_set;      // Normal power, 0x02 = 2 Hz
    IOCTL_arg.arg     = HIGH_PASS_2HZ;
    IOCTL_arg.channel = 1;
    IOCTL_result[3] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"GET HIGH PASS FILTER 2HZ \n");
    IOCTL_cmd.cmd = high_pass_get;
    IOCTL_result[4] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"SET HIGH PASS FILTER 8HZ \n");
    IOCTL_cmd.cmd = high_pass_set;
    IOCTL_arg.arg = HIGH_PASS_8HZ;
    IOCTL_arg.channel = 2;
    IOCTL_result[5] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"GET HIGH PASS FILTER 8HZ \n");
    IOCTL_cmd.cmd = high_pass_get;
    IOCTL_result[6] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    /*------------------------ Low pass filter  ------------------------------*/
    UART_PutString(USART2, (uint8_t*)"SET LOW PASS FILTER 45HZ \n");
    IOCTL_cmd.cmd = low_pass_set;
    IOCTL_arg.arg = LOW_PASS_45;
    IOCTL_result[7] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"GET LOW PASS FILTER 45HZ \n");
    IOCTL_cmd.cmd = low_pass_get;
    IOCTL_result[8] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    /*------------------------ Gain changes ----------------------------------*/
    UART_PutString(USART2, (uint8_t*)"SET GAIN TO 8X \n");
    IOCTL_cmd.cmd = gain_set;
    IOCTL_arg.arg = GAIN_08;
    IOCTL_arg.channel = 3;
    IOCTL_result[9] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"GET GAIN TO 8X \n");
    IOCTL_cmd.cmd = gain_get;
    IOCTL_result[10] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);

    UART_PutString(USART2, (uint8_t*)"SET GAIN TO 16X \n");
    IOCTL_cmd.cmd = gain_set;
    IOCTL_arg.arg = GAIN_16;
    IOCTL_arg.channel = 4;
    IOCTL_result[11] = IRPyro_ioctl(IRPyro_test_Sensor, &IOCTL_cmd, IOCTL_arg);
    IRPyro_test_single_sensor_read_sample(IRPyro_test_Sensor);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    op_result = check_results(IOCTL_result, IOCTL_TEST_NUMBER);
    if (op_result == 1)
    {
        UART_PutString(USART2, (uint8_t*)"PASSED \n");
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"FAILED WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------END OF MULTIPLE SENSOR READ-------------------------\n");
    return op_result;
}
/** end of 9.6.8
 * @}
 */

/**
 * @brief  Single TO-CAN read
 * @note  The TO-CAN is a four pin sensor with no CS nor INT lines.
 *        As a result the device is controlled only via I2C. Power control
 *        should be independently supplied.
 *
 *        This is important in two ways:
 *          -  Stabilization of the device should be allowed.
 *          -  I2C address collision should be controlled externally.
 *
 *        This is a self-contained test that should be called directly from an
 *        empty main() function
 */
void ut_TO_CAN_read(void)
{
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_TO_can = {0}; // Instance of sensor
    IRPyro_TO_can.read_mode = 1;                   // Polled mode reading
    IRPyro_TO_can.address   = 0x08;                // I2C address to configure
    IRPyro_TO_can.number_of_active_channels = 1;   // configures channel 2 for streamming
    /*-------------------------- board setup ---------------------------------*/
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"Single TO-CAN read \n");
    for(;;)
    {
        ut03_single_sensor_read(&IRPyro_TO_can);
    }
}
void ut_TO_CAN_read2(void)
{

    /*------------------- Memory allocation, initialization ------------------*/
    IRPyro_devices IRPyro_device = {0};
    /*----------------------- --- CONFIGURATION -----  -----------------------*/
    AFE_reg_type IRPyro_AFE_Master =
    {
        .S7_S0 = 0,
        .LP    = LOW_POWER_DISABLE,
        .HP    = HP_DISABLE,
        .C_LP  = LOW_PASS_180,
        .CLK_OUT = CLK_ENABLE,
        .SYNC = MASTER,
        .TEMP = 0,
        .INT  = INT_ENABLE
    };
    AFE_reg_type IRPyro_AFE_Slave =
    {
        .S7_S0 = 0,
        .LP    = LOW_POWER_DISABLE,
        .HP    = HP_DISABLE,
        .C_LP  = LOW_PASS_180,
        .CLK_OUT = CLK_DISABLE,
        .SYNC = SLAVE,
        .TEMP = 0,
        .INT  = INT_ENABLE
    };
    /*-----------------    ADDRESSES   ------------*/
    /* Assuming four sensors on an IRPyro Break out board or demo board*/
    IRPyro_device[0].INT_Port = 1;
    IRPyro_device[0].CS_Port  = 2;
    IRPyro_device[0].CS_Pin   = 1;
    IRPyro_device[0].INT_Pin  = 4;
    IRPyro_device[0].address  = 0x0A;

    IRPyro_device[1].CS_Port  = 2;
    IRPyro_device[1].INT_Port = 1;
    IRPyro_device[1].CS_Pin   = 5;
    IRPyro_device[1].INT_Pin  = 5;
    IRPyro_device[1].address  = 0x0B;

    /*-----------------    CONFIGURATION   ------------*/
    IRPyro_device[0].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK DISABLED
    IRPyro_device[0].read_mode       = 1;
    IRPyro_device[0].number_of_active_channels = 1;

    IRPyro_device[1].AFE_register    = IRPyro_AFE_Slave; // MASTER, INT, CLK DISABLED
    IRPyro_device[1].read_mode       = 1;
    IRPyro_device[1].number_of_active_channels = 1;


    UART_PutString(USART2, (uint8_t*)"IRPyro TO: READ DATA FROM MULTIPLE SENSORS and detect frames missing with the sensors CONFIGURED FOR INTERRUPT\n");
    IRPyro_demo_board(0);
    ut04_multiple_sensor_read(IRPyro_device, 4);


}
/******************************************************************************/
/** @defgroup UT_10_IRPyro Unit testing support functions
 * @brief  assorted board manipulation and comparison functions
 * @{
 */

/**
 * @brief Changes the state of all the CS lines as used on the IRPyro backplane board
 */
void IRPyro_demo_board(uint8_t value)
{
    CS_pin_set(2,1,value);
    CS_pin_set(2,3,value);
    CS_pin_set(2,4,value);
    CS_pin_set(2,5,value);
}
/**
 * @brief  Checks the results of a test for pass or fail by adding individual results
 * @param test_results    individual test result (0 fail / 1 sucess)
 * @param number_of_tests success result
 * @retval op_result 1 PASS/0 FAIL
 */
static uint8_t check_results(uint8_t * test_results, uint8_t number_of_tests)
{
    uint8_t op_result = 0;
    for (uint8_t idx=0; idx< number_of_tests; ++idx)
    {
        if (test_results[idx] == 1)
        {
            ++ op_result;
        }
        else
        {
            op_result = test_results[idx];
            break;
        }
    }
    if (op_result == number_of_tests)
    {
        op_result = 1;
    }
    return op_result;
}
/**
 * @brief  Given a list of devices collects a number of samples and determines lost frames
 * @param  this_list group of IRPyro configuration data
 * @param  list_size     number of devices on the list
 * @retval number_of_frames_lost
 */
static uint8_t IRPyro_test_multiple_sensor_read_sample(IRPyro_devices this_list, uint8_t list_size)
{
    uint16_t number_of_frames_lost[4] = {0};
    uint16_t previous_frame_count[4]  = {0};
    uint16_t present_frame_count[4]   = {0};
    uint8_t  reading_attempt = 0;
    uint8_t  op_result = 0;

    for(uint8_t device=0; device<list_size; ++device)
    {
        op_result = FIFO_reset(&this_list[device]);
        if (op_result != 1)
        {
            UART_PutString(USART2, (uint8_t*)"DEVICE NUMBER: ");
            UART_PutNumber(USART2, (uint32_t) device);
            UART_PutString(USART2, (uint8_t*)" FAILED TO RESET WITH CODE ");
            UART_PutNumber(USART2, (uint32_t) op_result);
            UART_PutString(USART2, (uint8_t*)" \n.");
        }
    }
    op_result = IRPyro_read_devices(this_list, list_size);
    if (op_result != 1)
    {
        UART_PutString(USART2, (uint8_t*)"FAILED TO READ WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    for(uint8_t device=0; device< IRPyro_LIST_SIZE; ++device)
    {
        present_frame_count[device]  = this_list[device].channel_value[IRPyro_FRAME_COUNT_LOCATION]; // FRAME_COUNT
        previous_frame_count[device] = this_list[device].channel_value[IRPyro_FRAME_COUNT_LOCATION]; // FRAME_COUNT
    }
    UART_PutString(USART2, (uint8_t*)"READING ");
    UART_PutNumber(USART2, NUMBER_OF_FRAMES_TO_COUNT);
    UART_PutString(USART2, (uint8_t*)" FRAMES \n");
    int sample_counter=0;
    while(sample_counter < NUMBER_OF_FRAMES_TO_COUNT)
    {
        op_result = IRPyro_read_devices(this_list, list_size);
        if (op_result == 1)
        {
            ++sample_counter;
            for(uint8_t device=0; device< IRPyro_LIST_SIZE; ++device)
            {
                uint16_t dif = 0;
                present_frame_count[device] = this_list[device].channel_value[IRPyro_FRAME_COUNT_LOCATION]; // FRAME_COUNT
                dif = present_frame_count[device] - previous_frame_count[device];
                if (dif > 1)
                {
                    ++number_of_frames_lost[device];
                }
                previous_frame_count[device] = present_frame_count[device];
            }
        }
        else
        {
            if (reading_attempt < 3)
            {
                ++reading_attempt;
            }
            else
            {
                ++sample_counter;
                for(uint8_t device=0; device< IRPyro_LIST_SIZE; ++device)
                {
                    ++number_of_frames_lost[device];
                }
                reading_attempt=0;
            }
        }
    }
    UART_PutString(USART2, (uint8_t*)"TEST COMPLETE  \n");
    UART_PutString(USART2, (uint8_t*)"Frames lost \n");
    int total;
    for(uint8_t device=0; device< IRPyro_LIST_SIZE; ++device)
    {
        total =+number_of_frames_lost[device];
        UART_PutString(USART2, (uint8_t*)"Device: ");
        UART_PutNumber(USART2, device);
        UART_PutString(USART2, (uint8_t*)"   ");
        UART_PutString(USART2, (uint8_t*)" frames lost: ");
        UART_PutNumber(USART2, number_of_frames_lost[device]);
        UART_PutString(USART2, (uint8_t*)"\n");
    }
    return (total == 0) ? 1 : 0;  // 1 passed all / 0 fail;
}
/**
 * @brief  IRPyro data collection for a number samples
 * @param  this_device IRPyro configuration data
 * @retval number_of_frames_lost
 */
static uint8_t IRPyro_test_single_sensor_read_sample(IRPyro_sensor_device_type* this_device)
{
    uint16_t number_of_frames_lost = 0;
    uint16_t previous_frame_count  = 0;
    uint16_t present_frame_count   = 0;
    uint8_t reading_attempt = 0;
    uint8_t op_result = 0;

    op_result = FIFO_reset(this_device);
    if (op_result != 1)
    {
        UART_PutString(USART2, (uint8_t*)"FAILED TO RESET WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    op_result = IRPyro_read(this_device);
    if (op_result != 1)
    {
        UART_PutString(USART2, (uint8_t*)"FAILED TO READ WITH CODE ");
        UART_PutNumber(USART2, (uint32_t) op_result);
        UART_PutString(USART2, (uint8_t*)" \n.");
    }
    previous_frame_count = this_device->channel_value[IRPyro_FRAME_COUNT_LOCATION];      // FRAME_COUNT
    present_frame_count  = this_device->channel_value[IRPyro_FRAME_COUNT_LOCATION];      // FRAME_COUNT

    UART_PutString(USART2, (uint8_t*)"READING ");
    UART_PutNumber(USART2, NUMBER_OF_FRAMES_TO_COUNT);
    UART_PutString(USART2, (uint8_t*)" FRAMES \n");
    uint32_t sample_counter=0;
    while(sample_counter < NUMBER_OF_FRAMES_TO_COUNT)
    {
        op_result = IRPyro_read(this_device);
        if (op_result == 1)
        {
            ++sample_counter;
            uint16_t dif = 0;
            present_frame_count = this_device->channel_value[IRPyro_FRAME_COUNT_LOCATION]; // FRAME_COUNT
            dif = present_frame_count - previous_frame_count;
            if (dif > 1)
            {
                ++number_of_frames_lost;
            }
            previous_frame_count = present_frame_count;
        }
        else
        {
            if (reading_attempt < 3)
            {
                ++reading_attempt;
            }
            else
            {
                ++sample_counter;
                ++number_of_frames_lost;
                reading_attempt=0;
            }
        }
    }
    UART_PutString(USART2, (uint8_t*)"TEST COMPLETE  \n");
    UART_PutString(USART2, (uint8_t*)"Frames lost ");
    UART_PutNumber(USART2, number_of_frames_lost);
    UART_PutString(USART2, (uint8_t*)"\n");
    return (number_of_frames_lost == 0) ? 1 : 0;  // 1 passed all / 0 fail;
}
/**
 * @brief  Compares AFE registers for equality
 * @param  first    First AFE
 * @param  second   Second AFE
 * @retval comparison 0 not equal / 1 equal
 */
static uint8_t AFE_compare(AFE_reg_type first, AFE_reg_type second)
{
    uint8_t i = 0, result_comparison = 1;
    uint8_t *AFE_pointer_one = &(first.S7_S0);
    uint8_t *AFE_pointer_two = &(second.S7_S0);
    uint8_t AFE_size = sizeof(first);

    while (result_comparison && i<AFE_size)
    {
        if (AFE_pointer_one[i] != AFE_pointer_two[i]  )
            result_comparison = 0;
        else
            ++i;
    }
    return result_comparison;
}
/**
 * @brief  Compares CCP registers for equality
 * @param  first    First CCP
 * @param  second   Second CCP
 * @retval comparison 0 not equal / 1 equal
 */
static uint8_t CCP_compare(CCP_reg_type first, CCP_reg_type second)
{
    uint8_t i = 0, result_comparison = 1;
    uint8_t *CCP_pointer_one = &(first.Feedback_cap);
    uint8_t *CCP_pointer_two = &(second.Feedback_cap);
    uint8_t CCP_size = sizeof(first);

    while (result_comparison && i<CCP_size)
    {
        if (CCP_pointer_one[i] != CCP_pointer_two[i]  )
            result_comparison = 0;
        else
            ++i;
    }
    return result_comparison;
}
/**
 * @brief  Compares WUP registers for equality
 * @param  first    First WUP
 * @param  second   Second WUP
 * @retval comparison 0 not equal / 1 equal
 */
static uint8_t WUP_compare(WUP_reg_type first, WUP_reg_type second)
{
    uint8_t i = 0, result_comparison = 1;
    uint8_t *WUP_pointer_one = &(first.UHT);
    uint8_t *WUP_pointer_two = &(second.UHT);
    uint8_t WUP_size = sizeof(first);

    while (result_comparison && i<WUP_size)
    {
        if (WUP_pointer_one[i] != WUP_pointer_two[i]  )
            result_comparison = 0;
        else
            ++i;
    }
    return result_comparison;
}
/**
  * @brief  Clear the entire FIFO (Rd/Wr pointer reset with the Empty)
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static  uint8_t FIFO_reset(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;

    i2c_result = i2c_Read(this_device->address, IRPyro_FIFO_RESET, &op_result ,1);
    if ((i2c_result == 1) && (op_result == IRPyro_FIFO_RESET_OK))
    {
        op_result =1;
    }
    else
        op_result = i2c_result;
    return op_result;
}
/**
 * @brief callback function
 * @attention This function is called from an interrupt service request function
 *            and should be minimal.
 */
static void wakeup_event_callback(void)
{
// add non blocking code
}
/** end of Unit testing support functions
 * @}
 */
/** end of Unit Test module
 * @}
 */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
